'use strict';

/* eslint-env jest */

var Metalsmith = require('metalsmith');
var equal = require('assert-dir-equal');
var rimraf = require('rimraf');
var path = require('path');
var plugin = require('./index');
var BuildEngine = require('../test/engines/build');
var ErrorEngine = require('../test/engines/error');
var OptionsEngine = require('../test/engines/options');

describe('metalsmith-in-place', function () {
  it('should process a single file', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'single-file');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin({ engine: BuildEngine })).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });

  it('should process multiple files', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'multiple-files');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin({ engine: BuildEngine })).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });

  it('should only process files that match the string pattern', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'string-pattern-process');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin({
      pattern: '*.md',
      engine: BuildEngine
    })).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });

  it('should only process files that match the array pattern', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'array-pattern-process');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin({
      pattern: ['index.md', 'extra.md'],
      engine: BuildEngine
    })).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });

  it('should fall back to the default engine', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'default-engine');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin()).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });

  it('should throw an error for invalid options', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'invalid-options');
    var metalsmith = new Metalsmith(base);

    return metalsmith.use(plugin({}, 'invalid option')).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should throw an error for an invalid pattern', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'invalid-pattern');
    var metalsmith = new Metalsmith(base);

    return metalsmith.use(plugin({
      pattern: function pattern() {}
    })).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should throw an error for an invalid engine', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'invalid-engine');
    var metalsmith = new Metalsmith(base);

    return metalsmith.use(plugin({
      engine: 'invalid engine'
    })).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should throw an error for an engine without a render method', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'missing-render');
    var metalsmith = new Metalsmith(base);

    return metalsmith.use(plugin({
      engine: function engine() {}
    })).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should throw an error for an engine with an invalid render property', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'invalid-render');
    var metalsmith = new Metalsmith(base);

    function Engine() {
      this.render = 'Invalid render prop';
    }

    return metalsmith.use(plugin({
      engine: Engine
    })).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should catch errors from the engine', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'engine-error');
    var metalsmith = new Metalsmith(base);

    return metalsmith.use(plugin({ engine: ErrorEngine })).build(function (err) {
      expect(err).toBeInstanceOf(Error);
      expect(err.message).toMatchSnapshot();
      done();
    });
  });

  it('should pass the engine options to the engine', function (done) {
    var base = path.join(process.cwd(), 'test', 'fixtures', 'engine-options');
    var actual = path.join(base, 'build');
    var expected = path.join(base, 'expected');
    var metalsmith = new Metalsmith(base);

    rimraf.sync(actual);

    return metalsmith.use(plugin({
      engine: OptionsEngine,
      engineOptions: {
        optionOne: 'value',
        optionTwo: 'value'
      }
    })).build(function (err) {
      if (err) {
        return done(err);
      }
      expect(function () {
        return equal(actual, expected);
      }).not.toThrow();
      return done();
    });
  });
});